@@ -1,8 +1,10 @@
 package URI::NamespaceMap;
 use Moose;
 use Moose::Util::TypeConstraints;
+use Module::Load::Conditional qw[can_load];
 use URI::Namespace;
-
+use Carp;
+use Try::Tiny;
 
 =head1 NAME
 
@@ -10,11 +12,11 @@ URI::NamespaceMap - Class holding a collection of namespaces
 
 =head1 VERSION
 
-Version 0.06
+Version 0.08
 
 =cut
 
-our $VERSION = '0.06';
+our $VERSION = '0.08';
 
 
 =head1 SYNOPSIS
@@ -36,11 +38,15 @@ This module provides an object to manage multiple namespaces for creating L<URI:
 
 =over
 
-=item C<< new ( [ \%namespaces ] ) >>
+=item C<< new ( [ \%namespaces | @prefixes | @uris ] ) >>
 
 Returns a new namespace map object. You can pass a hash reference with
 mappings from local names to namespace URIs (given as string or
-L<RDF::Trine::Node::Resource>) or namespaces_map with a hashref.
+L<RDF::Trine::Node::Resource>) or namespaces_map with a hashref. You
+may also pass an arrayref containing just prefixes and/or namespace URIs,
+and the module will try to guess the missing part. To use this
+feature, you need L<RDF::NS>, L<XML::CommonNS> or L<RDF::Prefixes>, or
+preferably all of them.
 
 
 =item C<< add_mapping ( $name => $uri ) >>
@@ -69,6 +75,9 @@ Returns an array of prefixes.
 
 around BUILDARGS => sub {
 	my ($next, $self, @parameters) = @_;
+	if (ref($parameters[0]) eq 'ARRAY') {
+		return { namespace_map => $self->_guess(@{$parameters[0]}) };
+	}
 	return $self->$next(@parameters) if (@parameters > 1);
 	return $self->$next(@parameters) if (exists $parameters[0]->{namespace_map});
 	return { namespace_map => $parameters[0] };
@@ -126,7 +135,7 @@ sub uri {
 	}
 	return unless (blessed($ns));
 	if ($local ne '') {
-		return $ns->$local();
+		return $ns->_uri($local);
 	} else {
 		return $ns->uri
 	}
@@ -242,6 +251,54 @@ sub AUTOLOAD {
 	return $ns;
 }
 
+sub _guess {
+	my ($self, @data) = @_;
+	my $xmlns = can_load( modules => { 'XML::CommonNS' => 0 } );
+	my $rdfns = can_load( modules => { 'RDF::NS' => 20130802 } );
+	my $rdfpr = can_load( modules => { 'RDF::Prefixes' => 0 } );
+
+	confess 'To resolve an array, you need either XML::CommonNS, RDF::NS or RDF::Prefixes' unless ($xmlns || $rdfns || $rdfpr);
+	my %namespaces;
+
+	foreach my $entry (@data) {
+
+		if ($entry =~ m/^[a-z]\w+$/i) {
+			# This is a prefix
+			warn "Cannot resolve '$entry' without XML::CommonNS or RDF::NS" unless ($xmlns || $rdfns);
+			if ($xmlns) {
+			  require XML::CommonNS;
+			  XML::CommonNS->import(':all');
+			  try {
+				 $namespaces{$entry} = XML::CommonNS->uri(uc($entry))->toString;
+			  }; # Then, XML::CommonNS doesn't have the prefix, which is OK, we just continue
+			}
+			if ((! $namespaces{$entry}) && $rdfns) {
+				my $ns = RDF::NS->new;
+				$namespaces{$entry} = $ns->SELECT($entry);
+			}
+			warn "Cannot resolve '$entry'" unless $namespaces{$entry};
+		} else {
+			# Lets assume a URI string
+			warn "Cannot resolve '$entry' without RDF::NS or RDF::Prefixes" unless ($rdfns || $rdfpr);
+			my $prefix;
+			if ($rdfns) {
+				my $ns = RDF::NS->new;
+				$prefix = $ns->PREFIX($entry);
+			}
+			if ((! $prefix) && ($rdfpr)) {
+				my $context = RDF::Prefixes->new;
+				$prefix = $context->get_prefix($entry);
+			}
+			unless ($prefix) {
+				warn "Cannot resolve '$entry'";
+			} else {
+				$namespaces{$prefix} = $entry;
+			}
+		}
+	}
+	return \%namespaces;
+}
+
 
 =back
 
@@ -275,7 +332,7 @@ You can find documentation for this module with the perldoc command.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2012 Gregory Todd Williams, Chris Prather and Kjetil Kjernsmo
+Copyright 2012-2014 Gregory Todd Williams, Chris Prather and Kjetil Kjernsmo
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
